@@ -13,8 +13,8 @@ nanotestn(nextstream())
 nanotestn(nextget("pid"))
 nanotestz(status()[["connections"]])
 nanotestz(status()[["daemons"]])
-nanotest(is.character(launch_remote("ws://[::1]:5555")))
-nanotesterr(launch_remote("ws://[::1]:5555", command = "echo", args = "invalid"), "must be an element")
+nanotesterr(launch_remote("ws://[::1]:5555", remote = remote_config(command = "echo", args = "invalid")), "must be an element")
+nanotesterr(launch_remote(c("tcp://localhost:5555", "tcp://localhost:6666", "tcp://localhost:7777"), remote = remote_config(command = "echo", args = list(c("test", "."), c("test", ".")))), "length of 'args'")
 nanotesterr(launch_local(1L), "requires daemons to be set")
 nanotesto(daemons(1L, dispatcher = FALSE))
 me <- mirai(mirai::mirai(), .timeout = 2000L)
@@ -83,16 +83,24 @@ nanotest(mp$data == "test2" || unresolved(mp$data))
 Sys.sleep(1L)
 nanotestz(status(.compute = "new")[["connections"]])
 nanotestz(daemons(0L, .compute = "new"))
+nanotesterr(daemons(n = 1, maxtasks = "100"), "'...' arguments")
 Sys.sleep(1L)
+nanotest(is.list(ssh_config("ssh://remotehost")))
+nanotesterr(ssh_config("ssh://remotehost", tunnel = TRUE), "must be called in the correct context")
 if (.Platform[["OS.type"]] != "windows") {
+  mlc <- launch_remote("ws://[::1]:5555")
+  nanotest(is.character(mlc))
+  nanotest(inherits(mlc, "miraiLaunchCmd"))
+  nanotestp(mlc)
   nanotest(is_mirai(m <- mirai(TRUE)))
-  nanotest(is.character(launch_remote("ws://[::1]:5555", rscript = "/usr/lib/R/bin/Rscript", command = "echo", args = c("Test out:", ., ">/dev/null"))))
+  nanotest(is.character(launch_remote("ws://[::1]:5555", remote = remote_config(command = "echo", args = list(c("Test out:", ".", ">/dev/null")), rscript = "/usr/lib/R/bin/Rscript"))))
+  nanotest(is.character(launch_remote("tcp://localhost:5555", remote = ssh_config(remotes = c("ssh://remotehost", "ssh://remotenode"), tunnel = TRUE, command = "echo"))))
   Sys.sleep(2.5)
   nanotestn(launch_local(mirai:::auto_tokenized_url(), .compute = "test"))
   Sys.sleep(1L)
-  nanotest(daemons(n = 2L, url = value <- "ws://:0", dispatcher = FALSE) != value)
+  nanotest(daemons(n = 2L, url = value <- "ws://:0", dispatcher = FALSE, remote = remote_config()) != value)
   nanotestz(daemons(0L))
-  Sys.sleep(0.5)
+  Sys.sleep(1L)
   nanotesto(daemons(url = "ws://:0", token = TRUE))
   nanotestz(daemons(0L))
   Sys.sleep(1L)
@@ -144,28 +152,35 @@ if (.Platform[["OS.type"]] != "windows") {
   }
   nanotestz(daemons(0))
   Sys.sleep(1L)
+  cl <- make_cluster(1)
+  nanotest(inherits(cl, "miraiCluster"))
+  nanotest(inherits(cl, "cluster"))
+  nanotest(length(cl) == 1L)
+  nanotest(is.character(launch_remote(cl)))
+  nanotest(is.character(launch_remote(cl[[1L]])))
+  nanotest(is.list(status(cl)))
+  nanotestn(stop_cluster(cl))
+  Sys.sleep(1L)
 }
 if (Sys.getenv("NOT_CRAN") == "true" && .Platform[["OS.type"]] != "windows") {
-  nanotesto(daemons(url = "wss://:0", token = TRUE, pass = "test"))
+  nanotesto(daemons(url = "wss://127.0.0.1:0", token = TRUE, pass = "test"))
   nanotestn(launch_local(1L))
+  Sys.sleep(1L)
   nanotest(grepl("CERTIFICATE", launch_remote(1L), fixed = TRUE))
   nanotesterr(launch_local(0:1), "out of bounds")
   nanotesterr(launch_remote(1:2), "out of bounds")
-  nanotest(is.character(nextget("tls")))
   nanotestz(daemons(0L))
   Sys.sleep(1L)
   option <- 15L
-  nanotesto(daemons(1, dispatcher = TRUE, maxtasks = 10L, timerstart = 1L, walltime = 2000L, seed = 1546, token = TRUE, lock = TRUE, cleanup = option))
+  nanotesto(daemons(1, dispatcher = TRUE, maxtasks = 10L, timerstart = 1L, walltime = 1000L, seed = 1546, token = TRUE, lock = TRUE, cleanup = option))
   Sys.sleep(1L)
   mq <- mirai("daemon", .timeout = 1000)
+  nanotestn(saisei(i = 1L))
   nanotest(call_mirai(mq)$data == "daemon" || is_error_value(mq$data))
+  mq <- mirai(Sys.sleep(1.5), .timeout = 500)
   dstatus <- status()[["daemons"]]
   nanotest(is.matrix(dstatus) || is_error_value(dstatus))
-  if (is.matrix(dstatus)) {
-    nanotest(is.integer(dstatus[, "online"]))
-    nanotest(is.integer(dstatus[, "instance"]))
-  }
-  nanotestn(saisei(i = 1L))
+  Sys.sleep(2L)
   nanotestz(daemons(0))
   Sys.sleep(1L)
 }
